#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Follow and Align
#      By:	sdmitch
#   Usage:	Plugins>SDM Tools>CorG Tool>Follow and Align
#	 	 Note:	Places selected component(s) along a selected path in random or a fixed sequence.
#						Spacing refers to the distance between components.
#
#    Date:	Dec 2014
#------------------------------------------------------------------------------------------------
require 'Sketchup'
# ------------------  MENU SETUP ---------------------- #
unless $sdm_tools_menu
	$sdm_tools_menu = UI.menu("Plugins").add_submenu("SDM Tools")
	$sdm_Edge_tools = $sdm_tools_menu.add_submenu("Edge Tool")
	$sdm_Face_tools = $sdm_tools_menu.add_submenu("Face Tool")
	$sdm_CorG_tools = $sdm_tools_menu.add_submenu("CorG Tool")
	$sdm_Misc_tools = $sdm_tools_menu.add_submenu("Misc Tool")
end
unless file_loaded?(__FILE__)
	$sdm_CorG_tools.add_item("Follow and Align") { SDM::Follow_Me_Align.main }
	file_loaded(__FILE__)
end
# ------------------------------------------------------ #
module SDM

	module Follow_Me_Align

		def self.main
			mod = Sketchup.active_model
			ent = mod.active_entities
			sel = mod.selection
			nxt_edg = sel.first
			start = nxt_edg.start; found=true
			start = nxt_edg.end if nxt_edg.end.edges.length > 1
			mod.start_operation "Follow & Align"
			cds=mod.definitions.reject{|cd|cd.image? || cd.group?}; rmv=[]
			cds.each{|cd| rmv<<cd.entities.grep(Sketchup::ComponentInstance)};
			rmv.flatten!; rmv.uniq!; rmv.each{|x| cds.delete(x.definition)}; rmv=[]
			cdn=cds.map{|cd|cd.name}; name=cdn[0]; spc||='1'.to_l;
			ans=inputbox(["Component:","    Spacing:"],[name,@spc],[cdn.join("|")],"Follow & Align")
			return unless ans
			name=ans[0]; n=cdn.index(name); cd = cds[n]; spc=ans[1].to_l;
			while found
				found=false;
				if start.edges.length > 1
					edges=start.edges;n=edges.index(nxt_edg);
					if n
						for i in 0...edges.length
							unless i==n || sel.contains?(edges[i])
								ang=edges[n].line[1].angle_between(edges[i].line[1])
								if  ang < 20.degrees || ang > 160.degrees
									sel.add edges[i];found=true;
									start = edges[i].other_vertex(start)
									nxt_edg = edges[i];
								end
							end
						end
					end
				end
			end
			
			sel.grep(Sketchup::Edge).each{|e|
				copies=(e.length/cd.bounds.width).ceil; puts "copies=#{copies}"
				scale=e.length/(copies*(cd.bounds.width+spc)); puts "scale=#{scale}"
				width=cd.bounds.width*scale
				tr1 = Geom::Transformation.scaling(scale,1.0,1.0)
				p,v=e.line;xa=v.axes[2];za=e.faces[0].normal;za.reverse! if za.z<0;ya=xa*za
				for i in 0...copies
					ci=ent.add_instance(cd,tr1)
					dist=(width+spc)*i;dist=spc/2.0 if i==0; pt=p.offset(v,dist)
					ci.transform! Geom::Transformation.new(xa,ya,za,pt)
				end
			}
			mod.commit_operation
		end
		
	end
	
end

